#!/bin/bash
SCRIPT_NAME="$(basename "$0")"
SCRIPT_DIR="$(dirname "$(readlink -f "$0")")"

OLD_CONFS=false
PREV_INSTALLED_HA=
HA_VERSION=

. ${SCRIPT_DIR}/.install_common.sh

UFM_HA_CONF_FILE=/usr/local/bin/ha_utils/ufm-ha.conf

usage() {
    cat << EOF
Usage: ${SCRIPT_NAME} [ --drbd-location <DIRECTORY> ] [ --drbd-disk <PARTITION> ] [ --product [ cyber-ai | enterprise | enterprise-appliance | telemetry ] ]

The ${SCRIPT_NAME} installs UFM HA plugin for UFM products (cyber-ai, enterprise, enterprise-appliance, or telemetry).

OPTIONS:
    -l | --drbd-location           Directory used by DRBD to mount and sync files.
    -d | --drbd-disk               Name of disk partition for DRBD, for example: /dev/sdd4.
    -p | --product                 Name of UFM product to run in HA mode. Must be either: cyber-ai, enterprise, enterprise-appliance, or telemetry.
    -h | --help                    Show this message.

EOF
}

parse_options() {

    short_opts="l:d:p:h"
    long_opts="drbd-location:,drbd-disk:,product:,help"
    options=$(getopt -n ${SCRIPT_NAME} -o ${short_opts} --long ${long_opts} -- "$@")
    if [ $? -ne 0 ]; then
      usage
      exit 1
    fi

    eval set -- "$options"
    while true
    do
        case "$1" in
            -l | --drbd-location)
                DRBD_LOCATION=$2
                shift 2
                ;;
            -d | --drbd-disk)
                DRBD_DISK=$2
                shift 2
                ;;
            -p | --product)
                PRODUCT=$2
                shift 2
                ;;
            -h | --help)
                usage
                exit 0
                ;;
            --)
                shift
                break
                ;;
        esac
    done

    if [[ -z ${PRODUCT} ]]; then
        echo "Error: missing product name!"
        usage
        exit 1
    else
        validate_product
    fi

    # For now, drbd-disk and drbd-location are not required in Cyber AI
    cyberai_products=("${PRODUCT_CYBER_AI}" "${PRODUCT_UFM_CYBER_AI}")
    if [[ ! " ${cyberai_products[*]} " =~ " ${PRODUCT} " ]]; then
        if [[ -z ${DRBD_LOCATION} ]]; then
            echo "Error: missing drbd-location!"
            usage
            exit 1
        else 
            validate_drbd_location
        fi

        if [[ -z ${DRBD_DISK} ]]; then
            echo "Error: missing drbd-disk!"
            usage
            exit 1
        else 
            validate_drbd_disk
        fi
    fi
}

#######################################################################
#                Installation Main Logic                              #
#######################################################################
parse_options "$@"

# For Appliance Skip Check Prereqs stage.
if [[ ! -f "/etc/ufm-release" ]]; then
    check_pre_reqs
    if [[ $? -ne 0 ]]; then
        echo "- ERROR - Requirements check failed. Exiting Installation!"
        exit 1
    fi
fi

echo "====================================" > ${INSTALL_LOG}
date >> ${INSTALL_LOG}
log_msg "===================================="
log_msg " [*] Installing UFM HA for ${PRODUCT} [*]"
log_msg " ----------------------------------"

check_version

# Preserve old configurations if exist
confs_current_node_ip=
if [ -f "${UFM_HA_CONF_FILE}" ]; then
    confs_current_node_ip=$(get_config_value "current_node_ip" "nodes" "${UFM_HA_CONF_FILE}")
fi

log_msg "[*] Preparing configuration files"
handle_product_settings

pushd ${SCRIPT_DIR}/scripts > /dev/null
trap cleanup_install EXIT

log_msg "[*] Disabling pcs UI"
disable_pcs_gui

log_msg "[*] Copying files"
copy_files

log_msg "[*] Save UFM Product Type"
set_config_value "type" "${PRODUCT}" "product" $UFM_HA_CONF_FILE


# old configuration (UFM-HA < 5.1.0-4) 
if [[ ${OLD_CONFS} = true ]]; then
    echo "Warning: There are major changes in the HA settings, please reconfigure the HA cluster prior to starting it."
else 
    if [ -n "$PREV_INSTALLED_HA" ] && [ -n "$HA_VERSION" ]; then
        pcs status &>/dev/null
        exit_code=$?
        if [[ $exit_code -eq 0 ]]; then
            set_config_value "version" "${HA_VERSION}" "product" $UFM_HA_CONF_FILE
        fi
        if [ -n "${confs_current_node_ip}" ]; then
            set_config_value "current_node_ip" "${confs_current_node_ip}" "nodes" $UFM_HA_CONF_FILE
        fi
    fi
fi

echo "Done. Please run: 'systemctl daemon-reload' to load new HA services"
echo "To See full installation logs please check: ${INSTALL_LOG}"
