#!/bin/bash

SCRIPT_NAME="$(basename "$0")"
SCRIPT_DIR="$(dirname "$(readlink -f "$0")")"


usage() {
    cat << EOF
Usage: ${SCRIPT_NAME} [OPTIONS]

${SCRIPT_NAME} configures UFM-HA cluster on both HA nodes. Run this script ONLY once on the HA master node.

[*] NOTE: ${SCRIPT_NAME} requires SSH trust established between HA nodes.

OPTIONS:
    --cluster-password            HA cluster password
    --master-primary-ip           Master node primary IP address
    --standby-primary-ip          Standby node primary IP address
    --master-secondary-ip         Master node secondary IP address
    --standby-secondary-ip        Standby node secondary IP address
    --virtual-ip                  HA Cluster virtual IP address
    --no-vip                      Do not create virtual IP resource.
                                  Mutual exclusive with virtual-ip.
    -h | --help                   Show this message.

EOF
}

NO_VIP=false


parse_options() {
    short_opts="h"
    long_opts="cluster-password:,master-primary-ip:,standby-primary-ip:,master-secondary-ip:,standby-secondary-ip:,virtual-ip:,help,no-vip"
    options=$(getopt -n ${SCRIPT_NAME} -o ${short_opts} --long ${long_opts} -- "$@")
    if [ $? -ne 0 ]; then
      usage
      exit 1
    fi

    eval set -- "$options"
    while true
    do
        case "$1" in
            --cluster-password)
                cluster_pwd=$2
                shift 2
                ;;
            --master-primary-ip)
                master_primary_ip=$2
                shift 2
                ;;
            --standby-primary-ip)
                standby_primary_ip=$2
                shift 2
                ;;
            --master-secondary-ip)
                master_secondary_ip=$2
                shift 2
                ;;
            --standby-secondary-ip)
                standby_secondary_ip=$2
                shift 2
                ;;
            --virtual-ip)
                virtual_ip=$2
                shift 2
                ;;
            --no-vip)
                NO_VIP=true
                shift 1
                ;;
            -h | --help)
                usage
                exit 0
                ;;
            --)
                shift
                break
                ;;
        esac
    done

    if [[ -z ${cluster_pwd} ]]; then
        echo "ERROR - Missing argument: cluster-password"
        exit 1
    fi

    if [[ -z ${master_primary_ip} ]]; then
        echo "ERROR - Missing argument: master-primary-ip"
        exit 1
    fi

    if [[ -z ${standby_primary_ip} ]]; then
        echo "ERROR - Missing argument: standby-primary-ip"
        exit 1
    fi

    if [[ ${NO_VIP} == true && -n "${virtual_ip}" ]]; then
        echo "ERROR - the options: --no-vip and --virtual-ip are mutual exclusive and you can use only one of them."
        exit 1
    fi

    if [[ ${NO_VIP} != true &&  -z ${virtual_ip} ]]; then
        echo "ERROR - Missing argument: virtual-ip"
        exit 1
    fi

}

configure_ha(){
    ip -6 route get ${standby_primary_ip} > /dev/null 2>&1
    result=$?
    if [[ ${result} -eq 0 ]]; then
        is_IPv6="-6"
    else
        is_IPv6=""
    fi

    standby_config_cmd="/usr/local/bin/ufm_ha_cluster config -r standby -e ${master_primary_ip} -l ${standby_primary_ip} -p ${cluster_pwd}"
    if [[ -n "${master_secondary_ip}" && -n "${standby_secondary_ip}" ]]; then
        standby_config_cmd="${standby_config_cmd} -E ${master_secondary_ip} -L ${standby_secondary_ip}"
    fi
    ssh ${is_IPv6} ${standby_primary_ip} "${standby_config_cmd}"
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to configure HA cluster on Standby node" && return ${exit_code}

    config_cmd="/usr/local/bin/ufm_ha_cluster config -r master -e ${standby_primary_ip} -l ${master_primary_ip} -p ${cluster_pwd}"
    if [[ -n "${master_secondary_ip}" && -n "${standby_secondary_ip}" ]]; then
        config_cmd="${config_cmd} -L ${master_secondary_ip} -E ${standby_secondary_ip}"
    fi
    if [[ ${NO_VIP} == true ]]; then
        config_cmd="${config_cmd} --no-vip"
    else
        config_cmd="${config_cmd} -i ${virtual_ip}"
    fi
    ${config_cmd}
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to configure HA cluster on Master node" && return ${exit_code}
}

parse_options "$@"

if [[ ${NO_VIP} == true ]]; then
    vip_message="NO-VIP"
else
    vip_message="${virtual_ip}"
fi

echo "==================="
echo " Configure HA Nodes"
echo "==================="
echo "Master IP:                 ${master_primary_ip}"
echo "Standby IP:                ${standby_primary_ip}"
echo "Master SECONDARY IP:       ${master_secondary_ip}"
echo "Standby SECONDARY IP:      ${standby_secondary_ip}"
echo "Virtual IP:                ${vip_message}"
configure_ha
