#!/bin/bash
SCRIPT_NAME="$(basename "$0")"
curr_dt=$(date +%Y%m%d_%H%M%S)
DRBD_RESOURCE="ha_data"
DRBD_RES_FILE="/etc/drbd.d/${DRBD_RESOURCE}.res"
COROSYNC_FILE="/etc/corosync/corosync.conf"
DRBD_PATCH='  startup {
    wfc-timeout 10;
    degr-wfc-timeout 10;
  }
  disk {
    on-io-error detach;
  }
  net {
    timeout 60;
    connect-int 10;
    ping-int 10;
    after-sb-0pri discard-zero-changes;
    after-sb-1pri discard-secondary;
    after-sb-2pri disconnect;
  }'

temp_file=""

usage() {
    cat << EOF
Usage: ${SCRIPT_NAME} [--help] --role master <options> | --role standby

The ${SCRIPT_NAME} modifies the DRBD and corosync configurations to support Pacemaker multi-link.

OPTIONS:
    --role                      Machine's role [master,standby].
    --master-secondary-ip       Secondary IP of master machine.
    --standby-secondary-ip      Secondary IP of standby machine.
    --force                     Apply patches forcefully.
    --help                      Show this message.
EOF
}

check_missing_attr(){
    attr=$1
    value=$2
    if [[ -z ${value} ]]; then
        echo "Error: missing attribute '${attr}'."
        usage
        exit 1
    fi
}

parse_options() {
    short_opts="h"
    long_opts="role:,master-secondary-ip:,standby-secondary-ip:,force,help"
    options=$(getopt -n ${SCRIPT_NAME} -o ${short_opts} --long ${long_opts} -- "$@")
    if [ $? -ne 0 ]; then
      usage
      exit 1
    fi

    eval set -- "$options"
    while true
    do
        case "$1" in
            --role)
                role=$2
                shift 2
                ;;
            --master-secondary-ip)
                master_secondary_ip=$2
                shift 2
                ;;
            --standby-secondary-ip)
                standby_secondary_ip=$2
                shift 2
                ;;
            --force)
                force_update=true
                shift
                ;;
            -h | --help)
                usage
                exit 0
                ;;
            --)
                shift
                break
                ;;
        esac
    done
    check_missing_attr role $role
    if [ "$role" != "master" ] &&  [ "$role" != "standby" ]; then
        echo "Error: machine's role '${role}' is not valid!"
        exit 1
    fi
    if [ "${role}" == "master" ]; then
        check_missing_attr master-secondary-ip $master_secondary_ip
        check_missing_attr standby-secondary-ip $standby_secondary_ip
    else
        if [ -n "${master_secondary_ip}" ]; then
            echo "Error: Cannot use --master-secondary-ip with standby role."
            usage
            exit 1
        fi
        if [ -n "${standby_secondary_ip}" ]; then
            echo "Error: Cannot use --standby-secondary-ip with standby role."
            usage
            exit 1
        fi
    
    fi

}

create_drbd_patch_file() {
    temp_file="/tmp/drbd_patch.res.$$"    
    printf "%s\n" "${DRBD_PATCH}" > ${temp_file}
}

modify_drbd_res_file() {
    create_drbd_patch_file
    # find the last line with '}' and insert the addition befor it
    line_no=$(grep -n '}' ${DRBD_RES_FILE} | tail -1 | awk -F: '{print $1}')
    line_no=$((line_no-1))
    # sed with r commnd reads a file and insert it after the line
    sed -i.${curr_dt} "${line_no}r ${temp_file}" ${DRBD_RES_FILE}
    rm -f "${temp_file}"
}

_add_ring1_addr() {
    ring0_lines=($(grep -nE "ring0_addr" ${COROSYNC_FILE} | awk -F: '{print $1}'))
    if [ "${#ring0_lines[@]}" -ne 2 ]; then
        echo "Error: Could not find Pacemaker Redundant Ring Protocol configuration - no changes are done. Please make sure HA is configured."
        return 1
    fi
    # The order of addresses is mandatory
    ring1_addresses=("$master_secondary_ip" "$standby_secondary_ip")
    corosync_patch_applied=$(check_corosync_patch_applied)
    if [ "$corosync_patch_applied" == 'true' ] ; then
        echo "Update corosync seconday IPs"
        ring1_lines=($(grep -nE "ring1_addr" ${COROSYNC_FILE} | awk -F: '{print $1}'))
        for i in "${!ring1_addresses[@]}"; do
            ring1_line_no=${ring1_lines[$i]}
            ring1_addr_line="\ \ \ \ \ \ \ \ ring1_addr: ${ring1_addresses[$i]}"
            sed -i "${ring1_line_no} c ${ring1_addr_line}" ${COROSYNC_FILE}
        done
    else 
        echo "Set corosync seconday IPs"
        for i in "${!ring1_addresses[@]}"; do
            ring0_line_no=$((${ring0_lines[$i]} + $i))
            ring1_addr_line="\ \ \ \ \ \ \ \ ring1_addr: ${ring1_addresses[$i]}"
            sed -i "${ring0_line_no} a ${ring1_addr_line}" ${COROSYNC_FILE}
        done
    fi
    return 0
}

_add_rrp_mode() {
    if grep -q "rrp_mode" "${COROSYNC_FILE}"; then
        echo "rrp_mode is already configured."
        return 1
    fi
    transport_full_line=$(grep -nE "transport" ${COROSYNC_FILE})
    transport_line_no=$(echo "$transport_full_line" | awk -F: '{print $1}')
    rrp_mode_line="\ \ \ \ rrp_mode: passive"
    sed -i "${transport_line_no} a ${rrp_mode_line}" ${COROSYNC_FILE}
    return 0
}

modify_corosync_config() {
    \cp -f "$COROSYNC_FILE" "$COROSYNC_FILE.orig"
    _add_ring1_addr
    _add_rrp_mode
}

check_corosync_patch_applied(){
    if grep -q "ring1_addr" "$COROSYNC_FILE"; then
        echo "true"
        return 0
    fi
    echo "false"
    return 1
}

check_drbd_patch_applied(){
    if grep -q "startup" "$DRBD_RES_FILE"; then
        echo "true"
        return 0
    fi
    echo "false"
    return 1
}

get_hostname_ip() {
  local addr="$1"
  result=$(host ${addr})
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  echo "${result}" | head -n1 | awk '{print $NF}'
  return $?
}

_check_corosync_configured_using_ipv4(){
    if grep ring0_addr "$COROSYNC_FILE" | grep -Eoq "([0-9]{1,3}\.){3}[0-9]{1,3}"; then
       echo "true"
    else
       echo "false"
    fi
}

_check_if_ip_used_in_corosync() {
    ip=$1
    is_ring0_ipv4=$(_check_corosync_configured_using_ipv4)
    ALL_CLUSTER_NODES=($(grep ring0_addr "$COROSYNC_FILE" | awk -F": " '{print $2}'))
    for node in "${ALL_CLUSTER_NODES[@]}"; do
        if [ "$is_ring0_ipv4" == 'true' ]; then
            node_ip=$node
            err_msg="Error: IP ${ip} is already used by HA"
        else
            node_ip=$(get_hostname_ip $node)
            err_msg="Error: Hostname of ${ip} is already used by HA"
        fi
        if [[ "${node_ip}" == "${ip}" ]]; then
            echo $err_msg
            return 1
        fi
    done
    return 0
}

check_ips() {
    # 1. check master_secondary_ip is one of the local IP addresses on master
    err_msg="Cannot find master secondary IP address ${master_secondary_ip} on local machine - please make sure that script is running on the master node."
    ip addr show | grep -q "$master_secondary_ip/"
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Error: ${err_msg}" && return ${exit_code}
    # 2. check standby_secondary_ip is not one of the local IP addresses on the master
    err_msg="The standby secondary IP address ${standby_secondary_ip} is configured on the local machine - please make sure that the standby secondary IP address belongs to the secondary node."
    ip addr show | grep -q "$standby_secondary_ip/"
    exit_code=$?; [ ${exit_code} -eq 0 ] && echo "Error: ${err_msg}" && return 1
    # 3. check standby_secondary_ip is not already set for ring0_addr (hostname/IP)
    _check_if_ip_used_in_corosync  $master_secondary_ip 
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    # 4. check master_secondary_ip is not already set for ring0_addr (hostname/IP)
    _check_if_ip_used_in_corosync  $standby_secondary_ip 
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    return 0
}

apply_drbd_patch() {
    drbd_patches_applied=$(check_drbd_patch_applied)
    if [ "$drbd_patches_applied" == 'true' ]; then
        echo "DRBD patch is already applied"
    else 
        \cp -f "$DRBD_RES_FILE" "$DRBD_RES_FILE.orig"
        modify_drbd_res_file
        drbdadm adjust ha_data       
    fi
}

apply_corosync_patch() {
    ufm_ha_cluster stop
    pcs property set maintenance-mode=true
    modify_corosync_config
    systemctl restart corosync
    pcs cluster sync
    exit_code=$?
    pcs property set maintenance-mode=false
    [ ${exit_code} -ne 0 ] && echo "Error: Failed to restart corosync service" && return ${exit_code}
    ufm_ha_cluster start
}

show_corosync_status(){
    echo "Corosync service status"
    systemctl status corosync
    echo "Corosync configurations"
    pcs cluster corosync
    echo "Shows current corosync links status"
    corosync-cfgtool -s
}

check_corosync_updated_with_multilink(){
    corosync_status=$(corosync-cfgtool -s)
    if echo $corosync_status | grep -q 'RING ID 1' ; then
        echo "true"
    else
        echo "false"
    fi
}

######
# MAIN
######
parse_options "$@"
apply_drbd_patch
corosync_patch_applied=$(check_corosync_patch_applied)
if [ "${role}" == "master" ]; then
    if [ "$corosync_patch_applied" == 'true' ] && [ -z "$force_update" ] ; then
        echo "Corosync patch is already applied, in order to update the master and standby secondary IP addresses, please run the script using --force option."
        exit 1
    else
        check_ips
        if [[ $? -ne 0 ]]; then
            echo "Error: IP requirements check failed. Exiting patch!"
            exit 1
        fi
        apply_corosync_patch
        show_corosync_status
        echo "To complete HA configuration, please make sure to run this bash script on the standby machine."
    fi 
else
    if [ "$corosync_patch_applied" != 'true' ]; then
        echo "Error: HA patch is not applied on master node."
        exit 1
    else 
        corosync_updated=$(check_corosync_updated_with_multilink)
        if [ "$corosync_updated" == "true" ] && [ -z "$force_update" ]; then
            echo "Corosync patch is already applied, in order to update corosync please run the script using --force option."
            exit 1
        else 
            systemctl restart corosync
            show_corosync_status
        fi
    fi
fi
