#!/bin/bash
# Copyright (C) Mellanox Technologies Ltd. 2001-2021.  ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Mellanox Technologies Ltd.
# (the "Company") and all right, title, and interest in and to the software product,
# including all associated intellectual property rights, are and shall
# remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.

DRBD_GLOBAL_CONF_FILE="/etc/drbd.d/global_common.conf"
RESOURCE_METADATA="resource_metada.txt"
read -d '' DRBD_GLOBAL_CONF << EOF
global {
  usage-count  yes;
}
common {
  disk {
    on-io-error   detach;
    no-disk-flushes ;
    no-disk-barrier;
    c-plan-ahead  0;
    c-fill-target 24M;
    c-min-rate    80M;
    c-max-rate    4G;
    resync-rate   4G;
  }
  net {
    protocol     C;
    max-buffers  36k;
    sndbuf-size  1024k ;
    rcvbuf-size  2048k;
  }
}
EOF

set_drbd_global_conf() {
  ${SUDO}mv "${DRBD_GLOBAL_CONF_FILE}" "${DRBD_GLOBAL_CONF_FILE}.orig"
  echo "$DRBD_GLOBAL_CONF" | ${SUDO}tee "${DRBD_GLOBAL_CONF_FILE}" > /dev/null
}

restore_drbd_global_conf() {
  echo "restore global drbd conf file"
  DRBD_GLOBAL_CONF_FILE_ORIG="${DRBD_GLOBAL_CONF_FILE}.orig"
  if [ -f "${DRBD_GLOBAL_CONF_FILE_ORIG}" ]; then
    ${SUDO}mv -f  "${DRBD_GLOBAL_CONF_FILE_ORIG}" "${DRBD_GLOBAL_CONF_FILE}"
  fi
  return $?
}

copy_drbd_resource_file() {
  echo "copy drbd resource file"
  local resource_name="$1"
  echo "${SUDO}cp -f /tmp/${resource_name}.res /etc/drbd.d/"
  ${SUDO}cp -f "/tmp/${resource_name}.res" /etc/drbd.d/
  return $?
}

remove_drbd_resource_files() {
  echo "remove drbd resource files"
  ${SUDO}rm -f /etc/drbd.d/*.res
  return $?
}

init_drbd_resource_backup() {
  backup_dir=$1
  echo "Creating Backup directory ${backup_dir}..."
  if [[ -d ${backup_dir} ]]; then
    ${SUDO}rm -rf ${backup_dir}
  fi
  ${SUDO}mkdir -p "${backup_dir}"
  cat /dev/null | ${SUDO}tee "${backup_dir}/${RESOURCE_METADATA}"
}

validate_partition() {
    local resource_dir=$1
    mounted_pids=$(lsof -t +D $resource_dir 2>/dev/null)
    if [ -n "$mounted_pids" ] ; then
        echo "The following PIDs prevent unmount of $resource_dir:" $mounted_pids
        echo "$resource_dir is busy on ${LOCAL_SYNC_IP}, trying cleanup with SIGKILL"
        kill -9 $mounted_pids
    fi
    mounted_pids=$(lsof -t +D $resource_dir 2>/dev/null)
    if [ -n "$mounted_pids" ] ; then
        echo "The following PIDs prevent unmount of $resource_dir:" $mounted_pids
        echo "$resource_dir is busy on ${LOCAL_SYNC_IP}, unmount failed"
        return 1
    fi

}

unmount_partition() {
  local disk=$1
  local resource_dir=$2
  local backup_dir=$3
  echo "unmount partition ${resource_dir}"
  validate_partition "${resource_dir}"
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  echo "$(basename ${resource_dir})=${resource_dir}" | ${SUDO}tee -a "${backup_dir}/${RESOURCE_METADATA}"
  echo "Copying Files from ${resource_dir} to ${backup_dir}"
  ${SUDO}cp -pr ${resource_dir} ${backup_dir}
  exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Failed to copy files from ${resource_dir} to ${backup_dir}" && return ${exit_code}

  echo "Deleting Files from ${resource_dir}"
  ${SUDO}rm -rf ${resource_dir}/*
  exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Failed to remove files from ${resource_dir}" && return ${exit_code}
  ls -al ${resource_dir}

  ${SUDO}umount ${resource_dir}
  #exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}

  ${SUDO}dd if=/dev/zero of=${disk} count=1 bs=10M
  return ${exit_code}
}

load_drbd_module() {
  echo "load drbd module"
  echo "modprobe drbd"
  ${SUDO}modprobe drbd
  exit_code=$?
  if [ $exit_code -ne 0 ]; then
    echo "Cannot load drbd module" 1>&2
    return 1
  fi
  return ${exit_code}
}

restore_backup_resources() {
  local backup_dir=$1
  while read md_line; do
    local src=$(echo "${md_line}" | awk -F= '{print $1}')
    local dst=$(echo "${md_line}" | awk -F= '{print $2}')
    ${SUDO}cp -pr ${backup_dir}/${src}/. ${dst}
  done <${backup_dir}/${RESOURCE_METADATA}
}

connect_drbd_resources() {
  role=$1
  if [ "${role}" == "master" ] ; then
    ${SUDO}drbdadm primary --force all
    ${SUDO}drbdadm disconnect all
    ${SUDO}drbdadm connect all
  else
    ${SUDO}drbdadm secondary all
    ${SUDO}drbdadm disconnect all
    ${SUDO}drbdadm -- --discard-my-data connect all
  fi
}

bringup_drbd_resources() {
  local primary="$1"
  local backup_dir="$2"
  local drbd_disk="$3"
  local drbd_device="$4"
  echo "bringup drbd resources"
  local ans_temp_file=$(mktemp drbd_ans_XXXXXX --suffix=.txt --tmpdir)
  printf 'yes\n%.0s' {1..100} >> ${ans_temp_file}
  ${SUDO}drbdadm create-md all < ${ans_temp_file}
  rm ${ans_temp_file}
  ${SUDO}drbdadm up all
  local node_role=
  [[ "${primary}" == "true" ]] && node_role="master"
  connect_drbd_resources "${node_role}"

  ${SUDO}drbdsetup status all --verbose --statistics

  if [ ${primary} == "true" ] ; then
    md_disk=${drbd_disk/\/dev/\/dev\/drbd\/by-disk}
    if [[ ! -f "$md_disk" ]]; then
      md_disk=${drbd_device}
    fi
    ${SUDO}mkfs.ext4 -E nodiscard ${md_disk}

    echo "mount ${md_disk} ${DRBD_LOCATION}"
    ${SUDO}mount ${md_disk} ${DRBD_LOCATION}
    exit_code=$?
    if [ $exit_code -ne 0 ]; then
      echo "Cannot mount ${md_disk}" 1>&2
      return 1
    fi

    restore_backup_resources ${backup_dir}

  fi
  ${SUDO}drbdsetup status all --verbose --statistics
}

unmount_drbd(){
  echo "Unmount ${DRBD_DEVICE}"
  if grep -qs "${DRBD_DEVICE}" /proc/mounts; then
    ${SUDO}umount ${DRBD_DEVICE}
    return $?
  fi
  return 0
}

bringdown_drbd_resources(){
  echo "bring down drbd resources"
  ${SUDO}drbdadm down all
  return $?
}

reset_drbd_connectivity(){
  node_role=$1
  _cstatus=$(drbdadm cstate ha_data)
  exit_code=$?
  if [ $exit_code != 0  ]; then
    echo "Check the network connectivity on the machine"
  else 
    connect_drbd_resources "${node_role}"
  fi
}
