#!/bin/bash

SCRIPT_NAME="$(basename "$0")"
SCRIPT_DIR="$(dirname "$(readlink -f "$0")")"

UFM_HA_CLUSTER=/usr/local/bin/ufm_ha_cluster
COROSYNC_CONF_FILE=/etc/corosync/corosync.conf
SINGLE_LINK_MODE=false

usage() {
    cat << EOF
Usage: ${SCRIPT_NAME} [OPTIONS]

${SCRIPT_NAME} replace Standby node on HA cluster. Run this script ONLY once on the HA Master node.

[*] NOTE: ${SCRIPT_NAME} requires SSH trust established between HA nodes.

OPTIONS:
    --cluster-password            HA cluster password.
                                  mandatory.
    --standby-primary-ip          New standby node primary IP address.
                                  mandatory.
    --standby-secondary-ip        New standby node secondary IP address.
                                  mandatory.
    -h | --help                   Show this message.

EOF
}

_check_missing_attr(){
    attr=$1
    value=$2
    if [ -z ${value} ]; then
        echo "Error: missing attribute '--${attr}'."
        usage
        exit 1
    fi
}

_check_corosync_conf_exist(){
    if [ -z ${COROSYNC_CONF_FILE} ]; then
        echo "Error: HA cluster is not configured or running the script not on master node."
        exit 1
    fi
}

parse_options() {
    short_opts="h"
    long_opts="cluster-password:,standby-primary-ip:,standby-secondary-ip:,help"
    options=$(getopt -n ${SCRIPT_NAME} -o ${short_opts} --long ${long_opts} -- "$@")
    if [ $? -ne 0 ]; then
        usage
        exit 1
    fi
    
    eval set -- "$options"
    while true
    do
        case "$1" in
            --cluster-password)
                cluster_pwd=$2
                shift 2
            ;;
            --standby-primary-ip)
                standby_primary_ip=$2
                shift 2
            ;;
            --standby-secondary-ip)
                standby_secondary_ip=$2
                shift 2
            ;;
            -h | --help)
                usage
                exit 0
            ;;
            --)
                shift
                break
            ;;
        esac
    done
    
    _check_corosync_conf_exist
    
    _check_missing_attr "cluster-password" ${cluster_pwd}
    _check_missing_attr "standby-primary-ip" ${standby_primary_ip}
    if grep -q "ring1_addr" ${COROSYNC_CONF_FILE}; then
        # When ring1_addr exist, then providing multilinks becomes mandatory.
        _check_missing_attr "standby-secondary-ip" ${standby_secondary_ip}
    else
        SINGLE_LINK_MODE=true
    fi
}

detach_old_standby_node(){
    ${UFM_HA_CLUSTER} detach
}

get_master_ip_addresses(){
    # Ensure the user not running the script on standby machine since after detach corosync conf file won't exist
    _check_corosync_conf_exist
    master_primary_ip=$(grep -oP 'ring0_addr:\s*\K.*\s*' ${COROSYNC_CONF_FILE} | head -n1)
    if [ -z ${master_primary_ip} ]; then
        echo "Error: Missing master primary IP address, please check that master node is configured."
        exit 1
    fi
    if [ ${SINGLE_LINK_MODE} == false ]; then
        master_secondary_ip=$(grep -oP 'ring1_addr:\s*\K.*\s*' ${COROSYNC_CONF_FILE} | head -n1)
    fi
}

configure_new_standby_node() {
    ip -6 route get ${standby_primary_ip} > /dev/null 2>&1
    result=$?
    [[ ${result} -eq 0 ]] && is_IPv6="-6" || is_IPv6=""
    
    standby_config_cmd="${UFM_HA_CLUSTER} config -r standby -e ${master_primary_ip} -l ${standby_primary_ip} -p ${cluster_pwd}"
    if [ ${SINGLE_LINK_MODE} == false ]; then
        standby_config_cmd="${standby_config_cmd} -E ${master_secondary_ip} -L ${standby_secondary_ip}"
    else
        standby_config_cmd="${standby_config_cmd} --enable-single-link"
    fi
    ssh ${is_IPv6} ${standby_primary_ip} "${standby_config_cmd}"
}

attach_new_standby_node(){
    attach_cmd="${UFM_HA_CLUSTER} attach -l ${master_primary_ip} -e ${standby_primary_ip} -p ${cluster_pwd}"
    if [ ${SINGLE_LINK_MODE} == false ]; then
        attach_cmd="${attach_cmd} -E ${standby_secondary_ip}"
    fi
    ${attach_cmd}
}


replace_ha(){
    cluster_nodes_count=$(grep nodeid ${COROSYNC_CONF_FILE} | wc -l)
    if [ $cluster_nodes_count -eq 2 ]; then
        detach_old_standby_node
        exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Error: Failed to disconnect Standby node from cluster" && return ${exit_code}
    fi
    
    get_master_ip_addresses
    configure_new_standby_node
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Error: Failed to configure HA cluster on Standby node" && return ${exit_code}
    
    attach_new_standby_node
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Error: Failed to replace the standby node in HA cluster." && return ${exit_code}
}

parse_options "$@"

echo "==================="
echo " Replace Standby Node"
echo "==================="
echo "Standby Primary IP:       ${standby_primary_ip}"
echo "Standby secondary IP:     ${standby_secondary_ip}"

replace_ha
