#!/bin/bash
# Copyright (C) 2013-2023 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
#
# This software product is a proprietary product of Nvidia Corporation and its affiliates
# (the "Company") and all right, title, and interest in and to the software
# product, including all associated intellectual property rights, are and
# shall remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.

# [*] Summary:
# This utility script configures UFM Enterprise configuration file (gv.cfg)
# with HA nodes GUIDs and IPs
#
# 21.03.2022 Muthanna Nairat
SCRIPT_NAME="$(basename "$0")"
SCRIPT_DIR="$(dirname "$(readlink -f "$0")")"
CLUSTER_UTILS=${SCRIPT_DIR}/ha_utils/cluster_utils.sh
HA_CONFIG_FILE=${SCRIPT_DIR}/ha_utils/ufm-ha.conf
IBHOSTS_CMD=/opt/ufm/opensm/sbin/ibhosts
ENABLE=1
DISABLE=0
mode=
local_hostname=
peer_hostname=

. ${CLUSTER_UTILS}

usage() {
    cat << EOF
Usage: ${SCRIPT_NAME} [[-e|--enable] [OPTIONS] | [-d|--disable]]

${SCRIPT_NAME} Enables/Disables UFM Enterprise HA features.

OPTIONS:
    -l | --local-hostname              Local node hostname (Mandatory)
    -p | --peer-hostname               Peer node hostname (Mandatory)

    -h | --help                   Show this message.

Example:
ufm_enterprise_utility.sh --enable --local-hostname ha-server01 --peer-hostname ha-server02

EOF
}

parse_args() {
    short_opts="edhl:p:"
    long_opts="enable,disable,local-hostname:,peer-hostname:,help"
    options=$(getopt -n ${SCRIPT_NAME} -o ${short_opts} --long ${long_opts} -- "$@")
    if [ $? -ne 0 ]; then
      usage
      exit 1
    fi

    eval set -- "$options"
    while true
    do
        case "$1" in
            -e | --enable)
                mode=${ENABLE}
                shift
                ;;
            -d | --disable)
                mode=${DISABLE}
                shift
                ;;
            -l | --local-hostname)
                local_hostname=$2
                shift 2
                ;;
            -p | --peer-hostname)
                peer_hostname=$2
                shift 2
                ;;
            -h | --help)
                usage
                exit 0
                ;;
            --)
                shift
                break
                ;;
        esac
    done

    if [[ -z ${mode} ]]; then
        echo "ERROR - Missing Operation: --enable or --disable"
        exit 1
    fi

    if [[ ${mode} == ${ENABLE} ]]; then
        if [[ -z ${local_hostname} ]]; then
            echo "ERROR - Missing argument: local-hostname"
            exit 1
        fi
        if [[ -z ${peer_hostname} ]]; then
            echo "ERROR - Missing argument: peer-hostname"
            exit 1
        fi

    fi

}

GetConfigParam() {
	local value
	if [ -f ${GENERAL_UFM_CONF} ]; then
		value=`cut -f1 -d"#" ${GENERAL_UFM_CONF} | awk -v name=$1 -F "=" '$1 ~ name{gsub("[ \t\f\n\r\v]", "", $2); print $2; exit}'`
	else
		value="unknown"
	fi

	echo ${value}
}

UpdCfg()
{
    filename=$1
    section=$2
    eval key=\$3
    eval value=\$4
    tmp_file=/tmp/`basename $1`.tmp
    if [ ! -f $filename -o ! -s $filename ];then
    	echo -e "[$section]\n$key = $value" > $filename
    else
        awk -v s=[$section] -v "k=$key" -v "v=$value" -v found_section="false" -v found_key="false" -v eof=$(wc -l < $filename) '{ FS="=";
            line=$0;
            if (substr($1,1,1) == "[")
    	    {
    	      section=$1
    	    };
    	    if (section==s)
    	    {
    	      found_section="true"
    	      key=$1;
    	      gsub(/^[[:space:]]*/,"",key);
    	      gsub(/[[:space:]]+$/,"",key);
    	      if(key==k)
    	      {
    	        found_key="true"
    	        line=sprintf("%s = %s", key,v)
    	      }
            }
        	else
        	{
        	  if (found_section=="true" && found_key=="false")
        	  {
        		printf("%s = %s\n", k, v)
        		found_key="true"
        	  }
           }
           print line
           if (NR==eof)
           {
             if (found_section=="false")
               print s;
             if (found_key=="false")
               printf("%s = %s\n", k, v);
           }
         }' $filename > $tmp_file
        if [ $? -eq 0 ];then
            \cp -f $tmp_file $filename
            \rm $tmp_file
        else
            return 1
        fi
    fi
}

get_ibdev2netdev(){
    # Check if ibdev2netdev exists (CentOS 8 stream will not include OFED)
    # Download user script ibdev2netdev if it's not installed.
    if [ ! -f /usr/sbin/ibdev2netdev ]; then
        wget https://raw.githubusercontent.com/Mellanox/container_scripts/master/ibdev2netdev -O /usr/sbin/ibdev2netdev
        if [ ! -f /usr/sbin/ibdev2netdev ]; then
            echo "-ERROR- Could not download or find /usr/sbin/ibdev2netdev"
            exit 1
        fi
        chmod +x /usr/sbin/ibdev2netdev
    fi
}

get_ibhosts(){
    # If we could not find local ibhosts command we check if ibhosts exists in system path
    if [[ ! -f ${IBHOSTS_CMD} ]]; then
        which ibhosts > /dev/null
        if [[ $? -eq 0 ]]; then
            IBHOSTS_CMD=$(which ibhosts)
        else
            echo "-ERROR- Could not find 'ibhosts' command on this machine!"
            exit 1
        fi
    fi
}

import_ha_settings(){
    settings_file="${GENERAL_SETTINGS_DIR}/ha_settings"

    if [[ -f ${settings_file} ]]; then
        echo "Importing HA Configuration settings"
        . ${settings_file}
    else
        echo "-ERROR- Missing HA settings file: ${settings_file}!"
        exit 1
    fi
}

get_node_guid(){
    node_name=$(echo $1 | cut -d "." -f 1)
    fabric_interface=$(GetConfigParam fabric_interface)

    # Get Port name associated with fabric_interface
    # Some FW versions show ibhosts output with HCA-1 and Other versions Show mlx5_0
    # To support both we will check which one exists in ibhosts output for this node
    mlx5_ib_port=$(/usr/sbin/ibdev2netdev | grep "${fabric_interface}" | awk -F ' ' '{print $1}')
    hca_ib_port="HCA-$((${mlx5_ib_port: -1} + 1))"

    # First try to get GUID of mlx5 port name format
    node_guid=$(${IBHOSTS_CMD} | grep "${node_name}" | grep "${mlx5_ib_port}" | awk -F ' ' '{print $3}')

    # If node_guid is still empty. The port name could be displayed using HCA-1 format.
    if [[ -z ${node_guid} ]]; then
        node_guid=$(${IBHOSTS_CMD} | grep "${node_name}" | grep "${hca_ib_port}" | awk -F ' ' '{print $3}')
    fi

    # Return the GUID without the 0x prefix
    echo "${node_guid:2}"
}

# Main function to enable HA features in UFM Enterprise
# This method extract Node GUIDs for both servers (Based on fabric_interface)
# And then constructs HA_nodes and update gv.cfg
enable_ha_features(){
    parse_ini "${HA_CONFIG_FILE}"
    read_scripts_conf "quiet"
    read_general_conf "quiet"
    import_ha_settings

    # Make sure we have required IB commands to extract GUIDs info
    get_ibdev2netdev
    get_ibhosts

    master_guid=$(get_node_guid ${local_hostname})
    if [[ -z ${master_guid} ]]; then
        echo "ERROR: Could not read port GUID for master node"
        exit 1
    fi
    master_ip=${ha_master_primary_ip}

    standby_guid=$(get_node_guid ${peer_hostname})
    if [[ -z ${master_guid} ]]; then
        echo "ERROR: Could not read port GUID for standby node"
        exit 1
    fi
    standby_ip=${ha_standby_primary_ip}

    # Constructing HA_nodes value
    ha_nodes_info="${master_guid}:${master_ip}:1,${standby_guid}:${standby_ip}:2"

    echo "Updating gv.cfg HA_Nodes with: ${ha_nodes_info}"
    if [[ ! -f ${GENERAL_UFM_CONF} ]]; then
        echo "ERROR: Could not find UFM Configuration file: ${GENERAL_UFM_CONF}"
        exit 1
    fi

    UpdCfg ${GENERAL_UFM_CONF} "Disabled" "HA_nodes" ${ha_nodes_info}
    if [[ $? -ne 0 ]]; then
        echo "ERROR: Could not set HA settings in UFM gv.cfg"
        exit 1
    fi

}

disable_ha_features(){
    parse_ini "${HA_CONFIG_FILE}"
    read_general_conf "quiet"

    # Clean gv.cfg
    echo "Cleaning HA_nodes value from UFM configuration gv.cfg..."
    UpdCfg ${GENERAL_UFM_CONF} "Disabled" "HA_nodes" ""
    if [[ $? -ne 0 ]]; then
        echo "ERROR: Could not remove HA settings from UFM gv.cfg"
        exit 1
    fi

}

parse_args "$@"
if [[ ${mode} == ${ENABLE} ]]; then
    enable_ha_features
elif [[ ${mode} == ${DISABLE} ]]; then
    disable_ha_features
else
    echo "-ERROR- Invalid Operation Mode. User --enable Or --disable."
    exit 1
fi
exit 0
