#!/bin/bash
# Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
#
# This software product is a proprietary product of Mellanox Technologies Ltd.
# (the "Company") and all right, title, and interest in and to the software product,
# including all associated intellectual property rights, are and shall
# remain exclusively with the Company.
#
# This software product is governed by the End User License Agreement
# provided with the software product.
# this file is collecting system dump from ufm, docker and host, cyber-ai and ufm-ha
# After script collect the results, it tar the results and put them in backup folder
#
# exit error:
# 1 - error with running commands, or creating directory

set -o noglob

SCRIPTNAME=${0##*/}
SCRIPTNAME="${SCRIPTNAME%.*}"

LOGFILE=sysdump_collector_logger.log
DATE=`date +%F_%H-%M-%S`
OUTNAME=`hostname`_${SCRIPTNAME}_${DATE}
OUTFOLDER=/tmp/

default_location="True"
collected_ufm_sysdump="False"


mkdir -p ${OUTNAME}
if [[ $? -ne 0 ]] ;then echo "Cannot create folder in $(pwd) to put the sysdump in. Please change directory and run the script again"; exit 1; fi


rm -f $LOGFILE >/dev/null 2>&1

dolog() {
  echo  >> ${LOGFILE}
	echo "`date +%F_%H-%M-%S`    $@" | tee -a ${LOGFILE}
}

output_header() {
	local command="$@"
cat  <<  EOF
############################################################
# ${DATE}
# ${command}
############################################################
EOF
}

ufm_docker_collect() {
  if [ -d /tmp/ufm_sysdump ]; then
    dolog "Detected UFM folder in temp, removing it to create a new."
    # removing the folder of ufm_sysdump if exists to copy items to there
    rm -rf /tmp/ufm_sysdump >/dev/null 2>&1 
  fi
  #UFM docker sysdump collect
  if docker container inspect ufm >/dev/null 2>&1; then #UFM running on docker
    #if we could activate this command, it means there is docker ufm running,
    #thus, we need to collect host information that missing and copy the docker info from the machine.
    dolog "Detected a running UFM docker container instance, collecting UFM sysdump from the container."
    docker exec -it ufm sh -c '/opt/ufm/scripts/vsysinfo -e -f /opt/ufm/scripts/vsysinfo.docker.conf -O /tmp/docker_sysdump > /dev/null'

    echo "Moving UFM docker sysdump results to output folder - ${OUTFOLDER}/${OUTNAME}" >> $LOGFILE
    docker cp ufm:/tmp/docker_sysdump.tar.gz ${OUTNAME}

    #running getting sysdump for the host of the docker
    dolog "Collecting UFM sysdump from host."
    /opt/ufm/files/scripts/vsysinfo -S all -e -f /opt/ufm/files/scripts/vsysinfo.host.conf -O /tmp/ufm_sysdump >> $LOGFILE
    
    # if we want to copy the output of the file to /opt/ufm/backup at the end or
    # the user ran on output flag -O the default location is True
    if [[ "$default_location" != "False" ]] ; then default_location="docker"; fi


  elif [ -f /opt/ufm/files/scripts/vsysinfo ]; then
    #running sysdump if docker is down, we want to collect it
    echo "UFM contianer could not be detected (probably it's down)." >> $LOGFILE
    dolog "Collect UFM sysdump from host (UFM container instance is not running)"
    /opt/ufm/files/scripts/vsysinfo -S all -e -f /opt/ufm/files/scripts/vsysinfo.conf -O /tmp/ufm_sysdump >> $LOGFILE

  elif [ -d /opt/ufm/scripts ]; then
    dolog "Detected UFM bare-metal installation on the machine."
    #Regular system dump from host machine
    /opt/ufm/scripts/vsysinfo -S all -e -O /tmp/ufm_sysdump >> $LOGFILE

    if [[ "$default_location" != "False" ]] ; then default_location="bare-metal"; fi
  else
    echo "Could not find ufm on docker or on machine, searching for ufm container that was closed" >> $LOGFILE
    if [ "$(docker ps -a -q -f name=ufm)" ]; then
      echo "Detected UFM container which is not running." >> $LOGFILE
    else
      echo "No UFM was found, and the machine does not have it in a container as well" >> $LOGFILE
    fi
    return 1
  fi
  
  echo "Moving result to output folder - ${OUTFOLDER}/${OUTNAME}" >> $LOGFILE
  collected_ufm_sysdump="True"
  mv /tmp/ufm_sysdump.tar.gz ${OUTNAME}
}

cyberai_collect() {
  #if the file exists it means cyber-ai installed.
  if [[ -f /root/git/cyber-ai/tools/ufm-cai-sysdump ]]; then
    dolog "Cyber-AI installed, collecting system dump from it"
    /root/git/cyber-ai/tools/ufm-cai-sysdump -a >> $LOGFILE
    if [[ $? -ne 0 ]] ; then exit 1; fi
    cyberai_sysdump_file=$(ls -Art1 /tmp/cyberai-sysdump_* | tail -n 1)
    mv /tmp/${cyberai_sysdump_file} ${OUTNAME}
  elif docker container inspect ufm-plugin-cyberai >/dev/null 2>&1; then
    dolog "Detected cyber-ai docker container instance, running sysdump on the docker"
    docker exec -it ufm-plugin-cyberai sh -c '/root/git/cyber-ai/tools/ufm-cai-sysdump -a > /dev/null'
    #copy from docker the file to the folder
    dolog "Copying latest result from the docker"
    file_name=$(docker exec ufm-plugin-cyberai ls -rt /tmp/cyberai-sysdump_* | tail -n 1)
    docker cp ufm-plugin-cyberai:${file_name} ${OUTNAME}
  fi
}
HA_collect() {
  #if the HA container is up call for sysdump and then collect it
  if [ -d /opt/ha_data ]; then
    dolog "Detected UFM HA deployment, collecting local HA data."
    #Regular system dump from host machine
    if [[ ! -f /etc/ufm/ufm-ha-sysdump.conf ]]; then
      echo "Could not find the configuration file, this isnt the latest version of HA. please update HA so the sysdump can be collected."
      return 1
    fi
    /usr/bin/vsysinfo -S all -e -f /etc/ufm/ufm-ha-sysdump.conf -O /tmp/HA_sysdump >> $LOGFILE
    if [[ "$collected_ufm_sysdump" == "False" ]]; then
      dolog "Detected Setup is a Standby, collecting also host information."
      /usr/bin/vsysinfo -S all -e -f /etc/ufm/vsysinfo.host.conf -O /tmp/ufm_sysdump >> $LOGFILE
    fi
  else
    dolog "HA is not installed, skipping"
    return 1
  fi
  echo "Moving result to output folder - ${OUTFOLDER}/${OUTNAME}" >> $LOGFILE
  mv /tmp/HA_sysdump.tar.gz ${OUTNAME}
  if [[ -f /tmp/ufm_sysdump.tar.gz ]]; then
    mv /tmp/ufm_sysdump.tar.gz ${OUTNAME}
  fi
}

create_tar_and_put() {
mv ${LOGFILE} ${OUTNAME}

tar cfz ${OUTNAME}.tar.gz ${OUTNAME}
rm -rf ${OUTNAME}
if [[ "$default_location" == "bare-metal" ]]; then
  mv -f ${OUTNAME}.tar.gz /opt/ufm/backup/
  OUTFOLDER="/opt/ufm/backup/"
  [[ $(ls /opt/ufm/backup/ | wc -l) > 5 ]] && echo "There are more than 5 backups in backup folder, you may want to remove them"
elif [[ "$default_location" == "docker" ]]; then
  docker cp ${OUTNAME}.tar.gz ufm:/opt/ufm/backup/ 
  if [[ $? -ne 0 ]] ; then dolog "cannot copy file to docker";exit 1; fi
  OUTFOLDER="$(pwd)"
else
  mv -f ${OUTNAME}.tar.gz ${OUTFOLDER} >/dev/null 2>&1
fi
}

#Create header as output
output_header "${SCRIPTNAME} $@" | tee -a ${LOGFILE}

while : ;do
	case "$1" in
		--output|-O|-o) OUTFOLDER="$2"; default_location="False"; shift 2;;
		*) break;;
	esac
done

#UFM sysdump collect, collect from docker as well
ufm_docker_collect
#Collect cyberai sysdump
cyberai_collect
#Collect HA sysdump, Master and standby
HA_collect

#create the tar and put it in the location it need to be
create_tar_and_put

echo "Archive ${OUTFOLDER}/${OUTNAME}.tar.gz created successfully"

